import sys

class CLI():
    def __init__(self, args=None):
        if args == None:
            self.args = sys.argv
        else:
            self.args = args

    def getOrDie(self, key):
        value = self.getOrDefault(key, None)
        if value == None:
            print("Error, '" + key + "' does not exist.")
            sys.exit(1)
        else:
            return value
        
    def getOrDefault(self, key, defaultValue):
        index = self.indexOf(key)
        if index > -1:
            if len(self.args) <= index+1:
                return ""
            else:
                return self.args[index+1]
        else:
            return defaultValue

    def indexOf(self, key):
        if key in self.args:
            return self.args.index(key)
        else:
            return -1




import sys
import json
import requests

class GraymetaClient():

    def __init__(self, server_url, api_key):
        self.SERVER_URL = server_url
        self.API_KEY = api_key
        self.HEADERS = { "Authorization": "Bearer " + self.API_KEY }

    def features(self):
        return self.get("/api/data/features")

    def harvest_item(self, location_id, gm_item_id):
        """
        POST /api/control/harvest
        {
        	"location_id": "{location_id}",
        	"item_id": "{item_id}",
        	"force": {force}
        }
        """
        url = "/api/control/harvest"
        data = { "location_id": location_id, "item_id": gm_item_id, "force": "true"}
        return self.post(url, data)

    def harvest_container(self, location_id, container_id):
        """
        POST /api/control/harvest
        {
        	"location_id": "{location_id}",
        	"container_id": "{container_id}",
        	"force": {force}
        }
        """
        url = "/api/control/harvest"
        data = { "location_id": location_id, "container_id": container_id, "force": True }
        return self.post(url, data)

    def get_gm_item_from_s3_key(self, s3_key):
        gm_item_id, location_id = self.get_gm_item_id_from_s3_key(s3_key)
        if gm_item_id != None:
            item_data = self.get_gm_item_id(str(location_id), str(container["id"]), str(filename))
            item = self.get_gm_item(gm_item_id)
            return item
        else:
            return None


    def get_gm_item_id_from_s3_key(self, s3_key):
        key = s3_key.replace("s3://", "")
        splits = key.split("/")
        bucket = splits[0]
        filename = splits[-1]

        locations = self.list_locations()
        location_id = locations["locations"][0]["id"]
        containers = self.list_enabled_containers()

        container = None
        for candidate_container in containers:
            if candidate_container["id"] == bucket:
                container = candidate_container
                break

        if container is None:
            print("No container found matching '" + bucket + "'")
            sys.exit(1)
        else:
            item_data = self.get_gm_item_id(str(location_id), str(container["id"]), str(filename))
            if item_data != None:
                gm_item_id = item_data["gm_item_id"]
                return gm_item_id, location_id
            else:
                print("No gm_item_id found.")
                return None, None


    def get_gm_item_id(self, location_id, container_id, item_id):
        url = self.SERVER_URL + "/api/control/item-id"
        data = { "location_id": location_id, "container_id": container_id, "item_id": item_id }
        data_str = json.dumps(data)
        headers = self.HEADERS

        r = requests.post(url, data=data_str, headers=headers)
        if r.status_code >= 200 and r.status_code <= 299:
            return r.json()
        else:
            return None

    def get_gm_item(self, gm_item_id):
        return self.get("/api/data/items/" + gm_item_id)

    def get_gm_item_v2(self, gm_item_id):
        return self.get("/files/" + gm_item_id + "/metadata2.json")

    def list_items(self, container_id):
        return self.get("/api/data/items")

    def delete_gm_item(self, gm_item_id):
        url = self.SERVER_URL + "/api/data/items/" + gm_item_id
        headers = self.HEADERS
        r = requests.delete(url, headers=headers)
        return r.json()

    def upload_stl(self, gm_item_id, stl_filename):
        url = self.SERVER_URL + "/api/data/items/" + gm_item_id + "/captions"
        headers = self.HEADERS
        files = { "caption_file": open(stl_filename, 'rb') }
        r = requests.post(url, files=files, headers=headers)
        return r.json()

    def list_location(self, location_id):
        return self.get("/api/data/locations/" + location_id)

    def list_locations(self):
        return self.get("/api/data/locations")

    def list_containers(self, location_id):
        return self.get("/api/data/locations/" + location_id + "/containers")

    def list_enabled_containers(self):
        """
        GET /api/data/containers/enabled
        """
        return self.get("/api/data/containers/enabled")

    def health(self):
        return self.get("/api/data/healthz")

    def activity(self):
        return self.get("/api/data/activity")

    def user(self):
        return self.get("/api/data/user")

    def platform(self):
        return self.get("/api/data/summary/platform")

    def search(self):
        data = {}
        return self.post("/api/data/search", data)

    def compilations(self):
        return self.get("/api/data/summary/compilations")

    def keyword_list_groups(self):
        return self.get("/api/data/keywords")

    def keyword_get_group(self, group_id):
        return self.get("/api/data/keyword-groups/" + group_id)

    def keyword_create_group(self, name, color):
        data = {"name": name, "color": color }
        return self.post("/api/data/keyword-groups", data)

    def keyword_delete_group(self, group_id):
        url = self.SERVER_URL + "/api/data/keyword-groups/" + group_id
        headers = self.HEADERS
        r = requests.delete(url, headers=headers)
        return r.json()

    def keyword_add_to_group(self, group_id, word):
        url = "/api/data/keywords/" + group_id
        data = {"word": word}
        return self.post(url, data)

    def keyword_remove_from_group(self, group_id, word):
        url = self.SERVER_URL + "/api/data/keywords/" + group_id + "?word=" +word
        headers = self.HEADERS
        r = requests.delete(url, headers=headers)
        return r.json()

    def get(self, partial_url):
        url = self.SERVER_URL + partial_url
        headers = self.HEADERS
        r = requests.get(url, headers=headers)
        return r.json()

    def post(self, partial_url, data):
        url = self.SERVER_URL + partial_url
        headers = self.HEADERS
        data_str = json.dumps(data)
        r = requests.post(url, data=data_str, headers=headers)
        return r.json()




import sys
import os
import json

# py2/3 imports fix
from .gmapi import GraymetaClient
from .cli import CLI

VERSION="1.10.21"
COMMAND="gm"

def usageAndDie():
    print("gm is a tool for querying a graymeta.com installation over https")
    print("https://github.com/simonski/gm-api-python")
    print("")
    print("Usage: ")
    print("")
    print("    gm command [arguments]")
    print("")
    server_url = os.environ.get("GRAYMETA_SERVER_URL") or None
    server_key = os.environ.get("GRAYMETA_API_KEY") or None
    ljust_value = 60

    if server_url:
        print("    GRAYMETA_SERVER_URL".ljust(ljust_value) + ": " + server_url)
    else:
        print("    GRAYMETA_SERVER_URL".ljust(ljust_value) + ": unset - please `export GRAYMETA_SERVER_URL=https://your-graymeta-server`")

    if server_key:
        print("    GRAYMETA_API_KEY".ljust(ljust_value) + ": xxxxxxxx")
    else:
        print("    GRAYMETA_API_KEY".ljust(ljust_value) + ": unset - please `export GRAYMETA_API_KEY=xxxxxxx`")

    print("")
    print("The commands are:")
    print("")
    print("    list_locations".ljust(ljust_value) + "- displays all locations")
    print("    list_location {location_id}".ljust(ljust_value) + "- gets information on a specific location")
    print("")

    print("    list_containers {location_id}".ljust(ljust_value) + "- displays enabled containers")
    print("    list_all_containers {location_id}".ljust(ljust_value) + "- displays all containers")

    print("    list_items".ljust(ljust_value) + "- displays all items")
    print("")
    print("    get_gm_item_id {location_id} {container_id} {item_id}".ljust(ljust_value) + "- gets the gm_item_id for ")
    print("    get_gm_item {gm_item_id}".ljust(ljust_value) + "- gets metadata for an item using the gm_item_id")
    print("")
    print("    get_gm_item_id_from_s3_key {s3_key}".ljust(ljust_value) + "- gets gm_item_id from an s3_key")
    print("    get_gm_item_from_s3_key {s3_key}".ljust(ljust_value) + "- gets metadata for an item using the s3_key")
    print("")
    print("    delete_gm_item {gm_item_id}".ljust(ljust_value) + "- deletes the metadata from graymeta")
    print("")
    print("    upload_stl {gm_item_id} {stl_filename} ".ljust(ljust_value) + "- uploads and associates an STL file with content")
    print("")

    print("    harvest_item_from_s3_key".ljust(ljust_value) + "- forces a harvest for an item via its S3 key")
    print("    harvest_item {location_id} {gm_item_id}".ljust(ljust_value) + "- forces a harvest for a specific item")
    print("    harvest_container {location_id} {container_id}".ljust(ljust_value) + "- forces a harvest for an entire container.")

    print("")
    print("    keyword".ljust(ljust_value) + "- uses the Graymeta Keywords API")
    print("")
    print("    health".ljust(ljust_value) + "- print current /api/data/healthz data.")
    print("    activity".ljust(ljust_value) + "- print current /api/data/activity data.")
    print("    version".ljust(ljust_value) + "- print current gmapi version number.")
    print("    get {URL}".ljust(ljust_value) + "- returns response from a GET.")
    print("")

    sys.exit(0)

def version():
    print(COMMAND + " client " + VERSION)

def main():

    if len(sys.argv) == 1:
        usageAndDie()

    command = sys.argv[1]
    if command == "version":
        version()
        sys.exit(1)

    server_url = os.environ.get("GRAYMETA_SERVER_URL") or None
    server_key = os.environ.get("GRAYMETA_API_KEY") or None

    if server_url  is None or server_url.strip() == "":
        print("Error, GRAYMETA_SERVER_URL is required.")
        sys.exit(1)

    if server_key is None or server_key.strip() == "":
        print("Error, GRAYMETA_API_KEY is required.")
        sys.exit(1)

    gm = GraymetaClient(server_url, server_key)

    if command == "upload_stl":
        gm_item_id = sys.argv[2]
        stl_filename = sys.argv[3]
        nicePrint(gm.upload_stl(gm_item_id, stl_filename))

    elif command == "features":
        nicePrint(gm.features())

    elif command == "keyword":
        cli = CLI(sys.argv)
        command = cli.getOrDie("keyword")
        if command == "list":
            nicePrint(gm.keyword_list_groups())
        elif command == "get":
            group_id = cli.getOrDie("-group_id")
            nicePrint(gm.keyword_get_group(group_id))

        elif command == "create_group":
            name = cli.getOrDie("-name")
            color = "#" + cli.getOrDie("-color")
            nicePrint(gm.keyword_create_group(name, color))
        elif command == "delete_group":
            group_id = cli.getOrDie("-group_id")
            nicePrint(gm.keyword_delete_group(group_id))
        elif command == "add_to_group":
            group_id = cli.getOrDie("-group_id")
            word = cli.getOrDie("-word")
            nicePrint(gm.keyword_add_to_group(group_id, word))
        elif command == "remove_from_group":
            group_id = cli.getOrDie("-group_id")
            word = cli.getOrDie("-word")
            nicePrint(gm.keyword_remove_from_group(group_id, word))
        else:
            print("gm keyword (list | get | create_group | delete_group | add_to_group | remove_from_group)")

    elif command == "list_locations":
        nicePrint(gm.list_locations())

    elif command == "list_location":
        location_id = sys.argv[2]
        nicePrint(gm.list_location(location_id))

    elif command == "list_all_containers":
        location_id = sys.argv[2]
        nicePrint(gm.list_containers(location_id))

    elif command == "list_containers":
        nicePrint(gm.list_enabled_containers())

    elif command == "harvest_container":
        location_id = sys.argv[2]
        container_id = sys.argv[3]
        nicePrint(gm.harvest_container(location_id, container_id))

    elif command == "harvest_item":
        location_id = sys.argv[2]
        gm_item_id = sys.argv[3]
        nicePrint(gm.harvest_item(location_id, gm_item_id))

    elif command == "harvest_item_from_s3_key":
        s3_key = sys.argv[2]
        gm_item_id, location_id = gm.get_gm_item_id_from_s3_key(s3_key)
        nicePrint(gm.harvest_item(location_id, gm_item_id))

    elif command == "get_gm_item_id_from_s3_key":
        s3_key = sys.argv[2]
        gm_item_id = gm.get_gm_item_id_from_s3_key(s3_key)
        print(gm_item_id)

    elif command == "get_gm_item_id":
        location_id = sys.argv[2]
        container_id = sys.argv[3]
        item_id = sys.argv[4]
        nicePrint(gm.get_gm_item_id(location_id, container_id, item_id))

    elif command == "get_gm_item_from_s3_key":
        s3_key = sys.argv[2]
        nicePrint(gm.get_gm_item_from_s3_key(s3_key))

    elif command == "get_gm_item":
        gm_item_id = sys.argv[2]
        nicePrint(gm.get_gm_item(gm_item_id))

    elif command == "get_gm_item_v2":
        gm_item_id = sys.argv[2]
        nicePrint(gm.get_gm_item_v2(gm_item_id))

    elif command == "delete_gm_item":
        gm_item_id = sys.argv[2]
        nicePrint(gm.delete_gm_item(gm_item_id))

    elif command == "health":
        nicePrint(gm.health())

    elif command == "activity":
        nicePrint(gm.activity())

    elif command == "user":
        nicePrint(gm.user())

    elif command == "platform":
        nicePrint(gm.platform())

    elif command == "compilations":
        nicePrint(gm.compilations())

    elif command == "search":
        nicePrint(gm.search())

    elif command == "list_items":
        nicePrint(gm.search())

    elif command == "list_item_ids":
        results = gm.search()
        print("ItemID".ljust(35)+"Last Harvested".ljust(27) + "Name".ljust(20))
        for entry in results["results"]:
            result = entry["result"]
            gm_item_id = result["_id"]
            container = result.get("stow_container_id") or "stow_container_id"
            name = result.get("name") or "name"
            last_modified = result.get("last_modified") or "no last modified."
            last_harvested = result.get("last_harvested") or "no last harvested."

            print(gm_item_id.ljust(35) + last_harvested.ljust(27) + (container+"/"+name).ljust(20))


    elif command == "get":
        partial_url = sys.argv[2]
        nicePrint(gm.get(partial_url))

    else:
        print("I don't know how to '" + command + "'")
        sys.exit(1)

def nicePrint(data):
    if data:
        print(json.dumps(data, indent=4))
    else:
        print("No data found.")



#!/usr/bin/env python
"""A setuptools based setup module.

See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='gmapi',
    test_suite='tests',
    version='1.10.21',
    description='Python bindings to Graymeta.com Platform API v.21+',
    long_description=long_description,
    url='https://github.com/simonski/gm-api-python',
    author='Simon Gauld',
    license='Apache 2',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 2.7',
    ],

    # What does your project relate to?
    keywords='Client API to Graymeta v.21',

    packages=find_packages(exclude=['contrib', 'docs', 'tests']),

    install_requires=['requests'],

    extras_require={
        'dev': ['check-manifest'],
        'test': ['coverage'],
    },

    entry_points={
        'console_scripts': [
            'gm=gmapi.code:main',
        ],
    },
)
